==> hermes-2.4.10.3.tar.gz: hermes-2.4.10.3/lib/hermes/rex_client/tasks/update_structure_task.rb <==
module Hermes
  module RexClient
    class UpdateStructureTask

      def update_entity(rex_entity, className, client = Hermes::RexClient::Client.new, dont_update_parent = true,  arbo_ids = [], already_updated_parent = false)
        fields = rex_entity['fields']
        return if fields['id_cristal'].blank?

        client.log_puts("Struture fields #{fields}")
        
        arbo_ids << fields['id_cristal']
        structure = className.constantize.find_by(id_cristal: fields['id_cristal'])
        
        if !structure
          client.log_puts("Creating new structure #{fields['id_cristal']}")
          structure = className.constantize.create(generate_structure_fields(fields, rex_entity['id']))
        else
          structure.update(generate_structure_fields(fields, rex_entity['id']))
          client.log_puts("Updating existant structure #{fields['id_cristal']} #{structure.id}")
        end
        
        if fields['id_parent_cristal'].blank? 
          structure.parent_id = nil 
        else
          if !arbo_ids.include?(fields['id_parent_cristal'])
            parent = className.constantize.find_by(id_cristal: fields['id_parent_cristal'])
            if !already_updated_parent && !parent
              parent_rex_entity = client.search_structures({"fields.id_cristal": "|#{fields['id_parent_cristal']}"}).first
              client.log_puts("Updating parent structure #{fields['id_parent_cristal']}") if parent_rex_entity
              parent = update_entity(parent_rex_entity, className, client, dont_update_parent, arbo_ids, true) if parent_rex_entity
            end
            structure.parent_id = parent && parent.id 
          end
        end
        
        structure.save!
        structure
      end

      def generate_structure_fields(fields , id)      
        {
          title: fields['app_comm'],
          id_rex: id,
          id_cristal: fields['id_cristal'],
          id_parent_cristal: fields['id_parent_cristal'],
          siret: fields['siret'].length == 14 ? fields['siret'] : nil,
        }
      end
    end
  end
end

==> hermes-5.6.0.12.tar.gz: hermes-5.6.0.12/lib/hermes/rex_client/tasks/update_structure_task.rb <==
module Hermes
  module RexClient
    class UpdateStructureTask

      def update_entity(rex_entity, className, client = Hermes::RexClient::Client.new, dont_update_parent = true,  arbo_ids = [], already_updated_parent = false)
        fields = rex_entity['fields']
        return if fields['id_cristal'].blank?

        client.log_puts("Struture fields #{fields}")
        
        arbo_ids << fields['id_cristal']
        structure = className.constantize.find_by(id_cristal: fields['id_cristal'])
        
        if !structure
          client.log_puts("Creating new structure #{fields['id_cristal']}")
          structure = className.constantize.create(generate_structure_fields(fields, rex_entity['id']))
          client.log_puts("End Creating new structure #{fields['id_cristal']}")
        else
          client.log_puts("Updating existant structure #{fields['id_cristal']} #{structure.id}")
          structure.set(generate_structure_fields(fields, rex_entity['id']).merge(updated_at: DateTime.now))
          client.log_puts("End Updating existant structure #{fields['id_cristal']} #{structure.id}")
        end
        
        if fields['id_parent_cristal'].blank? 
          structure.parent_id = nil 
        else
          if !arbo_ids.include?(fields['id_parent_cristal'])
            parent = className.constantize.find_by(id_cristal: fields['id_parent_cristal'])
            if !already_updated_parent && !parent
              client.log_puts("Updating parent structure #{fields['id_parent_cristal']}")
              parent_rex_entity = client.search_structures({"fields.id_cristal": "|#{fields['id_parent_cristal']}"}).first
              client.log_puts("Found parent rex")if parent_rex_entity
              parent = update_entity(parent_rex_entity, className, client, dont_update_parent, arbo_ids, true) if parent_rex_entity
            end
            structure.parent_id = parent && parent.id 
            client.log_puts("Affected new parent #{structure.parent_id}") 
          end
        end
        
        client.log_puts("Starting save structure: #{structure.id_cristal} changed : #{structure.changes}") 
        structure.save!
        client.log_puts("End save structure: #{structure.id_cristal}") 
        structure
      end

      def generate_structure_fields(fields , id)      
        {
          title: fields['app_comm'],
          id_rex: id,
          id_cristal: fields['id_cristal'],
          id_parent_cristal: fields['id_parent_cristal'],
          siret: fields['siret'].length == 14 ? fields['siret'] : nil,
        }
      end
    end
  end
end
